import org.graphstream.algorithm.generator.Generator;
import org.graphstream.algorithm.generator.RandomGenerator;
import org.graphstream.graph.Edge;
import org.graphstream.graph.Graph;
import org.graphstream.graph.Node;
import org.graphstream.graph.Path;
import org.graphstream.graph.implementations.SingleGraph;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Objects;

public class Dijkstra {


	/**
	 * Méthode de test
	 * @param args arguments
	 */
	public static void main(String[] args) {
		System.setProperty("org.graphstream.ui", "swing");
		Graph g = Dijkstra.getRandomGraph("random", 3, 25);
		Node source = g.getNode(0);
		Node dest = g.getNode(10);
		g.nodes().forEach(n -> {
			n.setAttribute("label", n.getId());
			n.setAttribute("text-alignment", "above");
		});

		// Dijkstra intégré
		org.graphstream.algorithm.Dijkstra d = new org.graphstream.algorithm.Dijkstra(org.graphstream.algorithm.Dijkstra.Element.EDGE, "result", "length");
		d.init(g);
		d.setSource(source);
		d.compute();
		System.out.println(d.getPath(dest));

		// Dijstra créé
		Path myPath = dijkstra(g, source, dest);
		System.out.println(myPath);

		g.display();
	}

	private static Path dijkstra(Graph g, Node source, Node destination) {
		assert source != null;
		assert destination != null;
		HashMap<Node, Node> previous = new HashMap<>();
		HashMap<Node, Double> distances = new HashMap<>();
		ArrayList<Node> nodes = new ArrayList<>();
		g.nodes().forEach(n -> {
			nodes.add(n);
			distances.put(n, Double.POSITIVE_INFINITY);
			previous.put(n, null);
		});
		distances.put(source, 0.0);
		while(!nodes.isEmpty()) {
			Node n = findMin(nodes, distances);
			if(n == null) break;
			nodes.remove(n);
			n.neighborNodes().forEach(nei -> updateDistances(distances, previous, n, nei));
		}
		ArrayList<Edge> pathList = new ArrayList<>();
		Node pathNode = destination;
		while(pathNode != source) {
			if(pathNode == null) break;
			Edge e = pathNode.getEdgeBetween(previous.get(pathNode));
			pathList.add(e);
			pathNode = previous.get(pathNode);
		}
		if(pathList.stream().allMatch(Objects::isNull)) return new Path();
		Collections.reverse(pathList);
		Path path = new Path();
		path.setRoot(source);
		for(Edge e : pathList) {
			path.add(e);
		}
		return path;
	}

	private static Node findMin(ArrayList<Node> nodes, HashMap<Node, Double> distances) {
		Double min = Double.POSITIVE_INFINITY;
		Node minNode = null;
		for(Node n : nodes) {
			Double dist = distances.get(n);
			if(dist < min) {
				min = dist;
				minNode = n;
			}
			if(min == 0.0) break;
		}
		return minNode;
	}

	private static void updateDistances(HashMap<Node, Double> distances, HashMap<Node, Node> previous, Node n1, Node n2) {
		Double distN1 = distances.get(n1);
		Double distN2 = distances.get(n2);
		Edge n1n2 = n1.getEdgeBetween(n2);
		assert n1n2 != null;
		Double lengthN1N2 = Double.parseDouble(n1n2.getAttribute("length").toString());
		if(distN2 > distN1 + lengthN1N2) {
			distances.put(n2, distances.get(n1) + lengthN1N2);
			previous.put(n2, n1);
		}
	}

	private static Graph getRandomGraph(String id, float averageDegree, int nodeCount) {
		Graph g = new SingleGraph(id);
		Generator gen = new RandomGenerator(averageDegree, false, false, "", "length");
		gen.addSink(g);
		gen.begin();
		for(int i = 0; i < nodeCount; ++i)
			gen.nextEvents();
		gen.end();
		gen.removeSink(g);
		return g;
	}

}
